@@ -1,31 +1,54 @@
 #
-# $Id: http.pm,v 1.33 1997/12/12 10:09:53 aas Exp $
+# $Id: http.pm,v 1.39 1998/02/12 22:24:11 aas Exp $
 
 package LWP::Protocol::http;
 
 require LWP::Debug;
-require LWP::Protocol;
-require LWP::Socket;
-require HTTP::Request;
 require HTTP::Response;
 require HTTP::Status;
+require IO::Socket;
+require IO::Select;
 
-use Carp ();
-
+require LWP::Protocol;
 @ISA = qw(LWP::Protocol);
 
 use strict;
-
-my $httpversion  = 'HTTP/1.0';     # for requests
-my $endl         = "\015\012";     # how lines should be terminated;
+my $CRLF         = "\015\012";     # how lines should be terminated;
 				   # "\r\n" is not correct on all systems, for
 				   # instance MacPerl defines it to "\012\015"
 
 sub _new_socket
 {
-    LWP::Socket->new;
+    my($self, $host, $port, $timeout) = @_;
+
+    local($^W) = 0;  # IO::Socket::INET can be noisy
+    my $sock = IO::Socket::INET->new(PeerAddr => $host,
+				     PeerPort => $port,
+				     Proto    => 'tcp',
+				     Timeout  => $timeout,
+				    );
+    unless ($sock) {
+	# IO::Socket::INET leaves additional error messages in $@
+	$@ =~ s/^.*?: //;
+	die "Can't connect to $host:$port ($@)";
+    }
+    $sock;
 }
 
+
+sub _check_sock
+{
+    #my($self, $req, $sock) = @_;
+}
+
+sub _get_sock_info
+{
+    my($self, $res, $sock) = @_;
+    $res->header("Client-Peer" =>
+		 $sock->peerhost . ":" . $sock->peerport);
+}
+
+
 sub request
 {
     my($self, $request, $proxy, $arg, $size, $timeout) = @_;
@@ -35,7 +58,7 @@ sub request
 
     # check method
     my $method = $request->method;
-    unless ($method =~ /^[A-Za-z0-9_!#\$%&'*+\-.^`|~]+$/) {     # HTTP token
+    unless ($method =~ /^[A-Za-z0-9_!\#\$%&\'*+\-.^\`|~]+$/) {  # HTTP token
 	return new HTTP::Response &HTTP::Status::RC_BAD_REQUEST,
 				  'Library does not allow method ' .
 				  "$method for 'http:' URLs";
@@ -57,32 +80,32 @@ sub request
 	$fullpath = $url->full_path;
     }
 
-    alarm($timeout) if $self->use_alarm and $timeout;
-
     # connect to remote site
-    my $socket = $self->_new_socket();
-    $socket->connect($host, $port);
+    my $socket = $self->_new_socket($host, $port, $timeout);
+    $self->_check_sock($request, $socket);
+	    
+    my $sel = IO::Select->new($socket) if $timeout;
 
-    my $request_line = "$method $fullpath $httpversion$endl";
+    my $request_line = "$method $fullpath HTTP/1.0$CRLF";
 
     # If we're sending content we *have* to specify a content length
     # otherwise the server won't know a messagebody is coming.
-
     my $content = $request->content;
 
     # All this mess because we want to support content as both scalar,
     # ref to scalar and ref to code.
     my $contRef;
-    if (defined $content){
+    if (defined $content) {
 	$contRef = ref($content) ? $content : \$content;
 	if (ref($contRef) eq 'SCALAR') {
-	    $request->header('Content-Length', length $$contRef)
-	      if length $$contRef;
+	    $request->header('Content-Length' => length $$contRef)
+	        if length $$contRef;
 	} elsif (ref($contRef) eq 'CODE') {
-	    Carp::croak('No Content-Length header for request with content')
+	    die 'No Content-Length header for request with code content'
 	      unless $request->header('Content-Length');
 	} else {
-	    Carp::croak("Illegal content in request ($content)");
+	    my $type = ref($contRef);
+	    die "Illegal content type ($type) in request";
 	}
     }
 
@@ -99,27 +122,50 @@ sub request
 	}
     }
 
-    $socket->write($request_line . $request->headers_as_string($endl) . $endl);
+    # we always assume that we are writeable
+    my $buf = $request_line . $request->headers_as_string($CRLF) . $CRLF;
+    {
+	die "write timeout" if $timeout && !$sel->can_write($timeout);
+	my $n = $socket->syswrite($buf, length($buf));
+	die $! unless defined($n);
+	die "short write" unless $n == length($buf);
+	LWP::Debug::conns($buf);
+    }
     if (defined $content) {
 	if (ref($contRef) eq 'CODE') {
-	    $socket->write($contRef, $timeout);
-	} else {
-	    $socket->write($$contRef, $timeout);
+	    while ( ($buf = &$contRef()), defined($buf) && length($buf)) {
+		die "write timeout" if $timeout && !$sel->can_write($timeout);
+		my $n = $socket->syswrite($buf, length($buf));
+		die $! unless defined($n);
+		die "short write" unless $n == length($buf);
+		LWP::Debug::conns($buf);
+	    }
+	} elsif (length($$contRef)) {
+	    die "write timeout" if $timeout && !$sel->can_write($timeout);
+	    my $n = $socket->syswrite($$contRef, length($$contRef));
+	    die $! unless defined($n);
+	    die "short write" unless $n == length($$contRef);
+	    LWP::Debug::conns($buf);
 	}
     }
 
     # read response line from server
     LWP::Debug::debug('reading response');
 
-    my $res = "";
-    my $buf = "";
     my $response;
+    $buf = '';
 
     # Inside this loop we will read the response line and all headers
     # found in the response.
-    while ($socket->read(\$buf, undef, $timeout)) {
-	$res .= $buf;
-	if ($res =~ s/^(HTTP\/\d+\.\d+)[ \t]+(\d+)[ \t]*([^\012]*)\012//) {
+    while (1) {
+	{
+	    die "read timeout" if $timeout && !$sel->can_read($timeout);
+	    my $n = $socket->sysread($buf, $size, length($buf));
+	    die $! unless defined($n);
+	    die "unexpected EOF before status line seen" unless $n;
+	    LWP::Debug::conns($buf);
+	}
+	if ($buf =~ s/^(HTTP\/\d+\.\d+)[ \t]+(\d+)[ \t]*([^\012]*)\012//) {
 	    # HTTP/1.0 response or better
 	    my($ver,$code,$msg) = ($1, $2, $3);
 	    $msg =~ s/\015$//;
@@ -129,18 +175,21 @@ sub request
 
 	    # ensure that we have read all headers.  The headers will be
 	    # terminated by two blank lines
-	    while ($res !~ /\015?\012\015?\012/) {
+	    while ($buf !~ /\015?\012\015?\012/) {
 		# must read more if we can...
-		LWP::Debug::debug("need more data for headers");
-		last unless $socket->read(\$buf, undef, $timeout);
-		$res .= $buf;
+		LWP::Debug::debug("need more header data");
+		die "read timeout" if $timeout && !$sel->can_read($timeout);
+		my $n = $socket->sysread($buf, $size, length($buf));
+		die $! unless defined($n);
+		die "unexpected EOF before all headers seen" unless $n;
+		#LWP::Debug::conns($buf);
 	    }
 
 	    # now we start parsing the headers.  The strategy is to
 	    # remove one line at a time from the beginning of the header
 	    # buffer ($res).
 	    my($key, $val);
-	    while ($res =~ s/([^\012]*)\012//) {
+	    while ($buf =~ s/([^\012]*)\012//) {
 		my $line = $1;
 
 		# if we need to restore as content when illegal headers
@@ -155,22 +204,24 @@ sub request
 		    ($key, $val) = ($1, $2);
 		} elsif ($line =~ /^\s+(.*)/) {
 		    unless ($key) {
-			LWP::Debug::debug("Illegal continuation header");
-			$res = "$save$res";
+			$response->header("Client-Warning" =>
+					 => "Illegal continuation header");
+			$buf = "$save$buf";
 			last;
 		    }
 		    $val .= " $1";
 		} else {
-		    LWP::Debug::debug("Illegal header '$line'");
-		    $res = "$save$res";
+		    $response->header("Client-Warning" =>
+				      "Illegal header '$line'");
+		    $buf = "$save$buf";
 		    last;
 		}
 	    }
 	    $response->push_header($key, $val) if $key;
 	    last;
 
-	} elsif ((length($res) >= 5 and $res !~ /^HTTP\//) or
-		 $res =~ /\012/ ) {
+	} elsif ((length($buf) >= 5 and $buf !~ /^HTTP\//) or
+		 $buf =~ /\012/ ) {
 	    # HTTP/0.9 or worse
 	    LWP::Debug::debug("HTTP/0.9 assume OK");
 	    $response = HTTP::Response->new(&HTTP::Status::RC_OK, "OK");
@@ -179,24 +230,27 @@ sub request
 
 	} else {
 	    # need more data
-	    LWP::Debug::debug("need more data to know which protocol");
+	    LWP::Debug::debug("need more status line data");
 	}
     };
-    die "Unexpected EOF" unless $response;
-
-    $socket->pushback($res) if length $res;
     $response->request($request);
+    $self->_get_sock_info($response, $socket);
 
-    # need to read content
-    alarm($timeout) if $self->use_alarm and $timeout;
 
+    my $usebuf = length($buf) > 0;
     $response = $self->collect($arg, $response, sub {
-	LWP::Debug::debug('Collecting');
-	my $content = '';
-	my $result = $socket->read(\$content, $size, $timeout);
-	return \$content;
+        if ($usebuf) {
+	    $usebuf = 0;
+	    return \$buf;
+	}
+	die "read timeout" if $timeout && !$sel->can_read($timeout);
+	my $n = $socket->sysread($buf, $size);
+	die $! unless defined($n);
+	#LWP::Debug::conns($buf);
+	return \$buf;
 	} );
-    $socket = undef;  # close it
+
+    $socket->close;
 
     $response;
 }
